/****************************************************************************
** Meta object code from reading C++ file 'playerslistmodel.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../QTfrontend/model/playerslistmodel.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'playerslistmodel.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PlayersListModel_t {
    QByteArrayData data[18];
    char stringdata0[198];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PlayersListModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PlayersListModel_t qt_meta_stringdata_PlayersListModel = {
    {
QT_MOC_LITERAL(0, 0, 16), // "PlayersListModel"
QT_MOC_LITERAL(1, 17, 9), // "nickAdded"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 4), // "nick"
QT_MOC_LITERAL(4, 33, 10), // "notifyNick"
QT_MOC_LITERAL(5, 44, 11), // "nickRemoved"
QT_MOC_LITERAL(6, 56, 14), // "nickAddedLobby"
QT_MOC_LITERAL(7, 71, 16), // "nickRemovedLobby"
QT_MOC_LITERAL(8, 88, 7), // "message"
QT_MOC_LITERAL(9, 96, 9), // "addPlayer"
QT_MOC_LITERAL(10, 106, 8), // "nickname"
QT_MOC_LITERAL(11, 115, 6), // "notify"
QT_MOC_LITERAL(12, 122, 12), // "removePlayer"
QT_MOC_LITERAL(13, 135, 3), // "msg"
QT_MOC_LITERAL(14, 139, 16), // "playerJoinedRoom"
QT_MOC_LITERAL(15, 156, 14), // "playerLeftRoom"
QT_MOC_LITERAL(16, 171, 14), // "resetRoomFlags"
QT_MOC_LITERAL(17, 186, 11) // "setNickname"

    },
    "PlayersListModel\0nickAdded\0\0nick\0"
    "notifyNick\0nickRemoved\0nickAddedLobby\0"
    "nickRemovedLobby\0message\0addPlayer\0"
    "nickname\0notify\0removePlayer\0msg\0"
    "playerJoinedRoom\0playerLeftRoom\0"
    "resetRoomFlags\0setNickname"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PlayersListModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       5,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   74,    2, 0x06 /* Public */,
       5,    1,   79,    2, 0x06 /* Public */,
       6,    2,   82,    2, 0x06 /* Public */,
       7,    1,   87,    2, 0x06 /* Public */,
       7,    2,   90,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       9,    2,   95,    2, 0x0a /* Public */,
      12,    2,  100,    2, 0x0a /* Public */,
      12,    1,  105,    2, 0x2a /* Public | MethodCloned */,
      14,    2,  108,    2, 0x0a /* Public */,
      15,    1,  113,    2, 0x0a /* Public */,
      16,    0,  116,    2, 0x0a /* Public */,
      17,    1,  117,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    3,    4,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    3,    4,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    3,    8,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,   10,   11,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   10,   13,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,   10,   11,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   10,

       0        // eod
};

void PlayersListModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PlayersListModel *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->nickAdded((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->nickRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->nickAddedLobby((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 3: _t->nickRemovedLobby((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->nickRemovedLobby((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 5: _t->addPlayer((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 6: _t->removePlayer((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 7: _t->removePlayer((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->playerJoinedRoom((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 9: _t->playerLeftRoom((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->resetRoomFlags(); break;
        case 11: _t->setNickname((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PlayersListModel::*)(const QString & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlayersListModel::nickAdded)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PlayersListModel::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlayersListModel::nickRemoved)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PlayersListModel::*)(const QString & , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlayersListModel::nickAddedLobby)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PlayersListModel::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlayersListModel::nickRemovedLobby)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (PlayersListModel::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PlayersListModel::nickRemovedLobby)) {
                *result = 4;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PlayersListModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractListModel::staticMetaObject>(),
    qt_meta_stringdata_PlayersListModel.data,
    qt_meta_data_PlayersListModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PlayersListModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlayersListModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PlayersListModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractListModel::qt_metacast(_clname);
}

int PlayersListModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractListModel::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void PlayersListModel::nickAdded(const QString & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PlayersListModel::nickRemoved(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PlayersListModel::nickAddedLobby(const QString & _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void PlayersListModel::nickRemovedLobby(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void PlayersListModel::nickRemovedLobby(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
