/****************************************************************************
** Meta object code from reading C++ file 'pageeditteam.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/page/pageeditteam.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pageeditteam.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PageEditTeam_t {
    QByteArrayData data[22];
    char stringdata0[280];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PageEditTeam_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PageEditTeam_t qt_meta_stringdata_PageEditTeam = {
    {
QT_MOC_LITERAL(0, 0, 12), // "PageEditTeam"
QT_MOC_LITERAL(1, 13, 19), // "CBTeamLvl_activated"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 5), // "index"
QT_MOC_LITERAL(4, 40, 16), // "CBFort_activated"
QT_MOC_LITERAL(5, 57, 21), // "frontendSoundsToggled"
QT_MOC_LITERAL(6, 79, 5), // "value"
QT_MOC_LITERAL(7, 85, 8), // "saveTeam"
QT_MOC_LITERAL(8, 94, 13), // "setRandomTeam"
QT_MOC_LITERAL(9, 108, 17), // "setRandomHogNames"
QT_MOC_LITERAL(10, 126, 13), // "setRandomHats"
QT_MOC_LITERAL(11, 140, 17), // "setRandomTeamName"
QT_MOC_LITERAL(12, 158, 14), // "setRandomGrave"
QT_MOC_LITERAL(13, 173, 13), // "setRandomFlag"
QT_MOC_LITERAL(14, 187, 14), // "setRandomVoice"
QT_MOC_LITERAL(15, 202, 13), // "setRandomFort"
QT_MOC_LITERAL(16, 216, 16), // "setRandomHogName"
QT_MOC_LITERAL(17, 233, 8), // "hh_index"
QT_MOC_LITERAL(18, 242, 9), // "testSound"
QT_MOC_LITERAL(19, 252, 9), // "fixHHname"
QT_MOC_LITERAL(20, 262, 3), // "idx"
QT_MOC_LITERAL(21, 266, 13) // "resetAllBinds"

    },
    "PageEditTeam\0CBTeamLvl_activated\0\0"
    "index\0CBFort_activated\0frontendSoundsToggled\0"
    "value\0saveTeam\0setRandomTeam\0"
    "setRandomHogNames\0setRandomHats\0"
    "setRandomTeamName\0setRandomGrave\0"
    "setRandomFlag\0setRandomVoice\0setRandomFort\0"
    "setRandomHogName\0hh_index\0testSound\0"
    "fixHHname\0idx\0resetAllBinds"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PageEditTeam[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   94,    2, 0x0a /* Public */,
       4,    1,   97,    2, 0x0a /* Public */,
       5,    1,  100,    2, 0x0a /* Public */,
       7,    0,  103,    2, 0x08 /* Private */,
       8,    0,  104,    2, 0x08 /* Private */,
       9,    0,  105,    2, 0x08 /* Private */,
      10,    0,  106,    2, 0x08 /* Private */,
      11,    0,  107,    2, 0x08 /* Private */,
      12,    0,  108,    2, 0x08 /* Private */,
      13,    0,  109,    2, 0x08 /* Private */,
      14,    0,  110,    2, 0x08 /* Private */,
      15,    0,  111,    2, 0x08 /* Private */,
      16,    1,  112,    2, 0x08 /* Private */,
      18,    0,  115,    2, 0x08 /* Private */,
      19,    1,  116,    2, 0x08 /* Private */,
      21,    0,  119,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Bool,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   17,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void,

       0        // eod
};

void PageEditTeam::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageEditTeam *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->CBTeamLvl_activated((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 1: _t->CBFort_activated((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 2: _t->frontendSoundsToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 3: _t->saveTeam(); break;
        case 4: _t->setRandomTeam(); break;
        case 5: _t->setRandomHogNames(); break;
        case 6: _t->setRandomHats(); break;
        case 7: _t->setRandomTeamName(); break;
        case 8: _t->setRandomGrave(); break;
        case 9: _t->setRandomFlag(); break;
        case 10: _t->setRandomVoice(); break;
        case 11: _t->setRandomFort(); break;
        case 12: _t->setRandomHogName((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->testSound(); break;
        case 14: _t->fixHHname((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->resetAllBinds(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PageEditTeam::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPage::staticMetaObject>(),
    qt_meta_stringdata_PageEditTeam.data,
    qt_meta_data_PageEditTeam,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PageEditTeam::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageEditTeam::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PageEditTeam.stringdata0))
        return static_cast<void*>(this);
    return AbstractPage::qt_metacast(_clname);
}

int PageEditTeam::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
