const char * creditsMessages[] = {
QT_TRANSLATE_NOOP("credits", "Programming"),
QT_TRANSLATE_NOOP("credits", "Game engine"),
QT_TRANSLATE_NOOP("credits", "Creator"),
QT_TRANSLATE_NOOP("credits", "Many engine improvements"),
QT_TRANSLATE_NOOP("credits", "Gamepad and Lua integration"),
QT_TRANSLATE_NOOP("credits", "Campaign support"),
QT_TRANSLATE_NOOP("credits", "Theme customization improvements"),
QT_TRANSLATE_NOOP("credits", "Some Pas2C and GLES2 work"),
QT_TRANSLATE_NOOP("credits", "Video recording"),
QT_TRANSLATE_NOOP("credits", "Other improvements"),
QT_TRANSLATE_NOOP("credits", "Map generation"),
QT_TRANSLATE_NOOP("credits", "Core map generators"),
QT_TRANSLATE_NOOP("credits", "Perlin maps and other improvements"),
QT_TRANSLATE_NOOP("credits", "Maze maps"),
QT_TRANSLATE_NOOP("credits", "Weapons"),
QT_TRANSLATE_NOOP("credits", "Most core weapons"),
QT_TRANSLATE_NOOP("credits", "Air mine, rubber, others"),
QT_TRANSLATE_NOOP("credits", "Drill rocket, ballgun, RC plane"),
QT_TRANSLATE_NOOP("credits", "Freezer"),
QT_TRANSLATE_NOOP("credits", "Mine number and time game settings"),
QT_TRANSLATE_NOOP("credits", "Frontend / main menu"),
QT_TRANSLATE_NOOP("credits", "Many frontend improvements"),
QT_TRANSLATE_NOOP("credits", "Keybinds, feedback, maps and hats interfaces"),
QT_TRANSLATE_NOOP("credits", "Login dialogs, other improvements"),
QT_TRANSLATE_NOOP("credits", "Missions and styles"),
QT_TRANSLATE_NOOP("credits", "A Classic Fairytale"),
QT_TRANSLATE_NOOP("credits", "A Space Adventure"),
QT_TRANSLATE_NOOP("credits", "Created Capture the Flag, Construction Mode, Control, HedgeEditor, Highlander, Racer, TechRacer, The Specialists, WxW"),
QT_TRANSLATE_NOOP("credits", "Training, time-trial and target practice challenges, Bazooka Battlefield, Tentacle Terror, Big Armory, bugfixes and maintenance"),
QT_TRANSLATE_NOOP("credits", "Some styles and missions"),
QT_TRANSLATE_NOOP("credits", "Battalion"),
QT_TRANSLATE_NOOP("credits", "Continental supplies"),
QT_TRANSLATE_NOOP("credits", "Teamwork 2"),
QT_TRANSLATE_NOOP("credits", "Climb Home"),
QT_TRANSLATE_NOOP("credits", "Portal Mind Challenge"),
QT_TRANSLATE_NOOP("credits", "Game server"),
QT_TRANSLATE_NOOP("credits", "Ports"),
QT_TRANSLATE_NOOP("credits", "macOS/iPhone port, OpenGL-ES conversion"),
QT_TRANSLATE_NOOP("credits", "Android port"),
QT_TRANSLATE_NOOP("credits", "Android netplay, portability abstraction"),
QT_TRANSLATE_NOOP("credits", "WebGL port"),
QT_TRANSLATE_NOOP("credits", "iPhone/iPad ports"),
QT_TRANSLATE_NOOP("credits", "Graphics"),
QT_TRANSLATE_NOOP("credits", "General"),
QT_TRANSLATE_NOOP("credits", "Themes"),
QT_TRANSLATE_NOOP("credits", "Nature, Snow, City, Castle, Halloween, Island"),
QT_TRANSLATE_NOOP("credits", "Bamboo, EarthRise, BambooPlinko"),
QT_TRANSLATE_NOOP("credits", "Golf, Hoggywood, Stage"),
QT_TRANSLATE_NOOP("credits", "Hoggywood"),
QT_TRANSLATE_NOOP("credits", "Cave, Olympics"),
QT_TRANSLATE_NOOP("credits", "Fruit, Cake"),
QT_TRANSLATE_NOOP("credits", "Art"),
QT_TRANSLATE_NOOP("credits", "Beach"),
QT_TRANSLATE_NOOP("credits", "Brick"),
QT_TRANSLATE_NOOP("credits", "Hell"),
QT_TRANSLATE_NOOP("credits", "Jungle"),
QT_TRANSLATE_NOOP("credits", "Sheep"),
QT_TRANSLATE_NOOP("credits", "Maps"),
QT_TRANSLATE_NOOP("credits", "Basketball, BasketballField, Bath, Bubbleflow, Hammock, Hedgelove, Hedgewars, Hydrant, Mushrooms, Plane, Ropes, Tree"),
QT_TRANSLATE_NOOP("credits", "SB_Bones, SB_Crystal, SB_Grassy, SB_Grove, SB_Haunty, SB_Oaks, SB_Shrooms, SB_Tentacle"),
QT_TRANSLATE_NOOP("credits", "Bamboo, Blox, Cake, Cogs, EarthRise, Freeway"),
QT_TRANSLATE_NOOP("credits", "Castle, PirateFlag"),
QT_TRANSLATE_NOOP("credits", "ShoppaKing, TrophyRace"),
QT_TRANSLATE_NOOP("credits", "Battlefield"),
QT_TRANSLATE_NOOP("credits", "CTF_Blizzard"),
QT_TRANSLATE_NOOP("credits", "Cheese"),
QT_TRANSLATE_NOOP("credits", "ClimbHome"),
QT_TRANSLATE_NOOP("credits", "Lonely_Island"),
QT_TRANSLATE_NOOP("credits", "Octorama"),
QT_TRANSLATE_NOOP("credits", "portal"),
QT_TRANSLATE_NOOP("credits", "Ruler"),
QT_TRANSLATE_NOOP("credits", "Sticks"),
QT_TRANSLATE_NOOP("credits", "Forts"),
QT_TRANSLATE_NOOP("credits", "EvilChicken"),
QT_TRANSLATE_NOOP("credits", "Olympic"),
QT_TRANSLATE_NOOP("credits", "Tank"),
QT_TRANSLATE_NOOP("credits", "Snail"),
QT_TRANSLATE_NOOP("credits", "SteelTower"),
QT_TRANSLATE_NOOP("credits", "Hats, graves, other"),
QT_TRANSLATE_NOOP("credits", "See CREDITS text file"),
QT_TRANSLATE_NOOP("credits", "Sounds"),
QT_TRANSLATE_NOOP("credits", "Hedgehogs voice"),
QT_TRANSLATE_NOOP("credits", "Default_pl, Russian_pl voices"),
QT_TRANSLATE_NOOP("credits", "Various authors from www.freesound.org (see CREDITS text file)"),
QT_TRANSLATE_NOOP("credits", "Music"),
QT_TRANSLATE_NOOP("credits", "City, Rock, others"),
QT_TRANSLATE_NOOP("credits", "Compost"),
QT_TRANSLATE_NOOP("credits", "EarthRise, oriental, Pirate, snow"),
QT_TRANSLATE_NOOP("credits", "Fruit, Jungle"),
QT_TRANSLATE_NOOP("credits", "Nature"),
QT_TRANSLATE_NOOP("credits", "olympics_sd"),
QT_TRANSLATE_NOOP("credits", "sdmusic (Hitman [sheepluva edit])"),
QT_TRANSLATE_NOOP("credits", "Translations"),
QT_TRANSLATE_NOOP("credits", "Brazilian Portuguese"),
QT_TRANSLATE_NOOP("credits", "Bulgarian"),
QT_TRANSLATE_NOOP("credits", "Czech"),
QT_TRANSLATE_NOOP("credits", "Chinese"),
QT_TRANSLATE_NOOP("credits", "Finnish"),
QT_TRANSLATE_NOOP("credits", "French"),
QT_TRANSLATE_NOOP("credits", "German"),
QT_TRANSLATE_NOOP("credits", "Greek"),
QT_TRANSLATE_NOOP("credits", "Hungarian"),
QT_TRANSLATE_NOOP("credits", "Italian"),
QT_TRANSLATE_NOOP("credits", "Japanese"),
QT_TRANSLATE_NOOP("credits", "Korean"),
QT_TRANSLATE_NOOP("credits", "Lithuanian"),
QT_TRANSLATE_NOOP("credits", "Polish"),
QT_TRANSLATE_NOOP("credits", "Portuguese"),
QT_TRANSLATE_NOOP("credits", "Russian"),
QT_TRANSLATE_NOOP("credits", "Scottish Gaelic"),
QT_TRANSLATE_NOOP("credits", "Slovak"),
QT_TRANSLATE_NOOP("credits", "Spanish"),
QT_TRANSLATE_NOOP("credits", "Swedish"),
QT_TRANSLATE_NOOP("credits", "Ukrainian"),
QT_TRANSLATE_NOOP("credits", "Special thanks"),
QT_TRANSLATE_NOOP("credits", "Project founder")
};
