/****************************************************************************
** Meta object code from reading C++ file 'hwform.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../QTfrontend/hwform.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'hwform.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_HWForm_t {
    QByteArrayData data[112];
    char stringdata0[1496];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HWForm_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HWForm_t qt_meta_stringdata_HWForm = {
    {
QT_MOC_LITERAL(0, 0, 6), // "HWForm"
QT_MOC_LITERAL(1, 7, 9), // "GoToSaves"
QT_MOC_LITERAL(2, 17, 0), // ""
QT_MOC_LITERAL(3, 18, 9), // "GoToDemos"
QT_MOC_LITERAL(4, 28, 7), // "GoToNet"
QT_MOC_LITERAL(5, 36, 8), // "GoToHelp"
QT_MOC_LITERAL(6, 45, 15), // "GoToEditWeapons"
QT_MOC_LITERAL(7, 61, 14), // "GoToNewWeapons"
QT_MOC_LITERAL(8, 76, 11), // "GoToWeapons"
QT_MOC_LITERAL(9, 88, 5), // "index"
QT_MOC_LITERAL(10, 94, 10), // "GoToScheme"
QT_MOC_LITERAL(11, 105, 14), // "GoToEditScheme"
QT_MOC_LITERAL(12, 120, 13), // "GoToNewScheme"
QT_MOC_LITERAL(13, 134, 8), // "GoToPage"
QT_MOC_LITERAL(14, 143, 2), // "id"
QT_MOC_LITERAL(15, 146, 6), // "GoBack"
QT_MOC_LITERAL(16, 153, 18), // "OpenSnapshotFolder"
QT_MOC_LITERAL(17, 172, 16), // "getDemoArguments"
QT_MOC_LITERAL(18, 189, 14), // "AssociateFiles"
QT_MOC_LITERAL(19, 204, 14), // "btnExitPressed"
QT_MOC_LITERAL(20, 219, 17), // "IntermediateSetup"
QT_MOC_LITERAL(21, 237, 7), // "NewTeam"
QT_MOC_LITERAL(22, 245, 8), // "EditTeam"
QT_MOC_LITERAL(23, 254, 8), // "teamName"
QT_MOC_LITERAL(24, 263, 13), // "AfterTeamEdit"
QT_MOC_LITERAL(25, 277, 10), // "DeleteTeam"
QT_MOC_LITERAL(26, 288, 12), // "DeleteScheme"
QT_MOC_LITERAL(27, 301, 15), // "DeleteWeaponSet"
QT_MOC_LITERAL(28, 317, 10), // "SimpleGame"
QT_MOC_LITERAL(29, 328, 8), // "PlayDemo"
QT_MOC_LITERAL(30, 337, 13), // "startTraining"
QT_MOC_LITERAL(31, 351, 13), // "StartCampaign"
QT_MOC_LITERAL(32, 365, 10), // "NetConnect"
QT_MOC_LITERAL(33, 376, 16), // "NetConnectServer"
QT_MOC_LITERAL(34, 393, 4), // "host"
QT_MOC_LITERAL(35, 398, 4), // "port"
QT_MOC_LITERAL(36, 403, 6), // "useTls"
QT_MOC_LITERAL(37, 410, 24), // "NetConnectOfficialServer"
QT_MOC_LITERAL(38, 435, 14), // "NetStartServer"
QT_MOC_LITERAL(39, 450, 13), // "NetRedirected"
QT_MOC_LITERAL(40, 464, 13), // "NetDisconnect"
QT_MOC_LITERAL(41, 478, 12), // "NetConnected"
QT_MOC_LITERAL(42, 491, 8), // "NetError"
QT_MOC_LITERAL(43, 500, 6), // "errmsg"
QT_MOC_LITERAL(44, 507, 10), // "NetWarning"
QT_MOC_LITERAL(45, 518, 6), // "wrnmsg"
QT_MOC_LITERAL(46, 525, 12), // "NetGameEnter"
QT_MOC_LITERAL(47, 538, 11), // "NetPassword"
QT_MOC_LITERAL(48, 550, 4), // "nick"
QT_MOC_LITERAL(49, 555, 17), // "NetNickRegistered"
QT_MOC_LITERAL(50, 573, 20), // "NetNickNotRegistered"
QT_MOC_LITERAL(51, 594, 12), // "NetNickTaken"
QT_MOC_LITERAL(52, 607, 13), // "NetAuthFailed"
QT_MOC_LITERAL(53, 621, 15), // "askRoomPassword"
QT_MOC_LITERAL(54, 637, 11), // "RetryDialog"
QT_MOC_LITERAL(55, 649, 5), // "title"
QT_MOC_LITERAL(56, 655, 5), // "label"
QT_MOC_LITERAL(57, 661, 15), // "NetTeamAccepted"
QT_MOC_LITERAL(58, 677, 4), // "team"
QT_MOC_LITERAL(59, 682, 10), // "AddNetTeam"
QT_MOC_LITERAL(60, 693, 6), // "HWTeam"
QT_MOC_LITERAL(61, 700, 13), // "RemoveNetTeam"
QT_MOC_LITERAL(62, 714, 11), // "StartMPGame"
QT_MOC_LITERAL(63, 726, 16), // "GameStateChanged"
QT_MOC_LITERAL(64, 743, 9), // "GameState"
QT_MOC_LITERAL(65, 753, 9), // "gameState"
QT_MOC_LITERAL(66, 763, 19), // "DemoPresenceChanged"
QT_MOC_LITERAL(67, 783, 7), // "hasDemo"
QT_MOC_LITERAL(68, 791, 16), // "ForcedDisconnect"
QT_MOC_LITERAL(69, 808, 6), // "reason"
QT_MOC_LITERAL(70, 815, 21), // "ShowFatalErrorMessage"
QT_MOC_LITERAL(71, 837, 9), // "GetRecord"
QT_MOC_LITERAL(72, 847, 10), // "RecordType"
QT_MOC_LITERAL(73, 858, 4), // "type"
QT_MOC_LITERAL(74, 863, 6), // "record"
QT_MOC_LITERAL(75, 870, 13), // "CreateNetGame"
QT_MOC_LITERAL(76, 884, 22), // "PlayOfficialServerDemo"
QT_MOC_LITERAL(77, 907, 13), // "UpdateWeapons"
QT_MOC_LITERAL(78, 921, 13), // "DeleteWeapons"
QT_MOC_LITERAL(79, 935, 11), // "weaponsName"
QT_MOC_LITERAL(80, 947, 10), // "AddWeapons"
QT_MOC_LITERAL(81, 958, 4), // "ammo"
QT_MOC_LITERAL(82, 963, 11), // "EditWeapons"
QT_MOC_LITERAL(83, 975, 14), // "oldWeaponsName"
QT_MOC_LITERAL(84, 990, 14), // "newWeaponsName"
QT_MOC_LITERAL(85, 1005, 20), // "onFrontendFullscreen"
QT_MOC_LITERAL(86, 1026, 5), // "value"
QT_MOC_LITERAL(87, 1032, 17), // "onFrontendEffects"
QT_MOC_LITERAL(88, 1050, 23), // "onFrontendSoundsToggled"
QT_MOC_LITERAL(89, 1074, 5), // "Music"
QT_MOC_LITERAL(90, 1080, 7), // "checked"
QT_MOC_LITERAL(91, 1088, 18), // "UpdateCampaignPage"
QT_MOC_LITERAL(92, 1107, 22), // "UpdateCampaignPageTeam"
QT_MOC_LITERAL(93, 1130, 26), // "UpdateCampaignPageProgress"
QT_MOC_LITERAL(94, 1157, 25), // "UpdateCampaignPageMission"
QT_MOC_LITERAL(95, 1183, 22), // "UpdateTrainingPageTeam"
QT_MOC_LITERAL(96, 1206, 16), // "InitCampaignPage"
QT_MOC_LITERAL(97, 1223, 32), // "RestoreSingleplayerTeamSelection"
QT_MOC_LITERAL(98, 1256, 18), // "showFeedbackDialog"
QT_MOC_LITERAL(99, 1275, 28), // "showFeedbackDialogNetChecked"
QT_MOC_LITERAL(100, 1304, 19), // "NetGameChangeStatus"
QT_MOC_LITERAL(101, 1324, 8), // "isMaster"
QT_MOC_LITERAL(102, 1333, 13), // "NetGameMaster"
QT_MOC_LITERAL(103, 1347, 12), // "NetGameSlave"
QT_MOC_LITERAL(104, 1360, 19), // "AsyncNetServerStart"
QT_MOC_LITERAL(105, 1380, 11), // "NetLeftRoom"
QT_MOC_LITERAL(106, 1392, 20), // "selectFirstNetScheme"
QT_MOC_LITERAL(107, 1413, 22), // "saveDemoWithCustomName"
QT_MOC_LITERAL(108, 1436, 20), // "openRegistrationPage"
QT_MOC_LITERAL(109, 1457, 9), // "startGame"
QT_MOC_LITERAL(110, 1467, 11), // "restartGame"
QT_MOC_LITERAL(111, 1479, 16) // "FromNetProxySlot"

    },
    "HWForm\0GoToSaves\0\0GoToDemos\0GoToNet\0"
    "GoToHelp\0GoToEditWeapons\0GoToNewWeapons\0"
    "GoToWeapons\0index\0GoToScheme\0"
    "GoToEditScheme\0GoToNewScheme\0GoToPage\0"
    "id\0GoBack\0OpenSnapshotFolder\0"
    "getDemoArguments\0AssociateFiles\0"
    "btnExitPressed\0IntermediateSetup\0"
    "NewTeam\0EditTeam\0teamName\0AfterTeamEdit\0"
    "DeleteTeam\0DeleteScheme\0DeleteWeaponSet\0"
    "SimpleGame\0PlayDemo\0startTraining\0"
    "StartCampaign\0NetConnect\0NetConnectServer\0"
    "host\0port\0useTls\0NetConnectOfficialServer\0"
    "NetStartServer\0NetRedirected\0NetDisconnect\0"
    "NetConnected\0NetError\0errmsg\0NetWarning\0"
    "wrnmsg\0NetGameEnter\0NetPassword\0nick\0"
    "NetNickRegistered\0NetNickNotRegistered\0"
    "NetNickTaken\0NetAuthFailed\0askRoomPassword\0"
    "RetryDialog\0title\0label\0NetTeamAccepted\0"
    "team\0AddNetTeam\0HWTeam\0RemoveNetTeam\0"
    "StartMPGame\0GameStateChanged\0GameState\0"
    "gameState\0DemoPresenceChanged\0hasDemo\0"
    "ForcedDisconnect\0reason\0ShowFatalErrorMessage\0"
    "GetRecord\0RecordType\0type\0record\0"
    "CreateNetGame\0PlayOfficialServerDemo\0"
    "UpdateWeapons\0DeleteWeapons\0weaponsName\0"
    "AddWeapons\0ammo\0EditWeapons\0oldWeaponsName\0"
    "newWeaponsName\0onFrontendFullscreen\0"
    "value\0onFrontendEffects\0onFrontendSoundsToggled\0"
    "Music\0checked\0UpdateCampaignPage\0"
    "UpdateCampaignPageTeam\0"
    "UpdateCampaignPageProgress\0"
    "UpdateCampaignPageMission\0"
    "UpdateTrainingPageTeam\0InitCampaignPage\0"
    "RestoreSingleplayerTeamSelection\0"
    "showFeedbackDialog\0showFeedbackDialogNetChecked\0"
    "NetGameChangeStatus\0isMaster\0NetGameMaster\0"
    "NetGameSlave\0AsyncNetServerStart\0"
    "NetLeftRoom\0selectFirstNetScheme\0"
    "saveDemoWithCustomName\0openRegistrationPage\0"
    "startGame\0restartGame\0FromNetProxySlot"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HWForm[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      83,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  429,    2, 0x08 /* Private */,
       3,    0,  430,    2, 0x08 /* Private */,
       4,    0,  431,    2, 0x08 /* Private */,
       5,    0,  432,    2, 0x08 /* Private */,
       6,    0,  433,    2, 0x08 /* Private */,
       7,    0,  434,    2, 0x08 /* Private */,
       8,    1,  435,    2, 0x08 /* Private */,
      10,    1,  438,    2, 0x08 /* Private */,
      11,    0,  441,    2, 0x08 /* Private */,
      12,    0,  442,    2, 0x08 /* Private */,
      13,    1,  443,    2, 0x08 /* Private */,
      15,    0,  446,    2, 0x08 /* Private */,
      16,    0,  447,    2, 0x08 /* Private */,
      17,    0,  448,    2, 0x08 /* Private */,
      18,    0,  449,    2, 0x08 /* Private */,
      19,    0,  450,    2, 0x08 /* Private */,
      20,    0,  451,    2, 0x08 /* Private */,
      21,    0,  452,    2, 0x08 /* Private */,
      22,    1,  453,    2, 0x08 /* Private */,
      24,    0,  456,    2, 0x08 /* Private */,
      25,    1,  457,    2, 0x08 /* Private */,
      26,    0,  460,    2, 0x08 /* Private */,
      27,    0,  461,    2, 0x08 /* Private */,
      28,    0,  462,    2, 0x08 /* Private */,
      29,    0,  463,    2, 0x08 /* Private */,
      30,    2,  464,    2, 0x08 /* Private */,
      31,    0,  469,    2, 0x08 /* Private */,
      32,    0,  470,    2, 0x08 /* Private */,
      33,    3,  471,    2, 0x08 /* Private */,
      37,    0,  478,    2, 0x08 /* Private */,
      38,    0,  479,    2, 0x08 /* Private */,
      39,    1,  480,    2, 0x08 /* Private */,
      40,    0,  483,    2, 0x08 /* Private */,
      41,    0,  484,    2, 0x08 /* Private */,
      42,    1,  485,    2, 0x08 /* Private */,
      44,    1,  488,    2, 0x08 /* Private */,
      46,    0,  491,    2, 0x08 /* Private */,
      47,    1,  492,    2, 0x08 /* Private */,
      49,    1,  495,    2, 0x08 /* Private */,
      50,    1,  498,    2, 0x08 /* Private */,
      51,    1,  501,    2, 0x08 /* Private */,
      52,    0,  504,    2, 0x08 /* Private */,
      53,    0,  505,    2, 0x08 /* Private */,
      54,    2,  506,    2, 0x08 /* Private */,
      57,    1,  511,    2, 0x08 /* Private */,
      59,    1,  514,    2, 0x08 /* Private */,
      61,    1,  517,    2, 0x08 /* Private */,
      62,    0,  520,    2, 0x08 /* Private */,
      63,    1,  521,    2, 0x08 /* Private */,
      66,    1,  524,    2, 0x08 /* Private */,
      68,    1,  527,    2, 0x08 /* Private */,
      70,    1,  530,    2, 0x08 /* Private */,
      71,    2,  533,    2, 0x08 /* Private */,
      75,    0,  538,    2, 0x08 /* Private */,
      76,    0,  539,    2, 0x08 /* Private */,
      77,    0,  540,    2, 0x08 /* Private */,
      78,    1,  541,    2, 0x08 /* Private */,
      80,    2,  544,    2, 0x08 /* Private */,
      82,    3,  549,    2, 0x08 /* Private */,
      85,    1,  556,    2, 0x08 /* Private */,
      87,    1,  559,    2, 0x08 /* Private */,
      88,    1,  562,    2, 0x08 /* Private */,
      89,    1,  565,    2, 0x08 /* Private */,
      91,    1,  568,    2, 0x08 /* Private */,
      92,    1,  571,    2, 0x08 /* Private */,
      93,    1,  574,    2, 0x08 /* Private */,
      94,    1,  577,    2, 0x08 /* Private */,
      95,    1,  580,    2, 0x08 /* Private */,
      96,    0,  583,    2, 0x08 /* Private */,
      97,    0,  584,    2, 0x08 /* Private */,
      98,    0,  585,    2, 0x08 /* Private */,
      99,    0,  586,    2, 0x08 /* Private */,
     100,    1,  587,    2, 0x08 /* Private */,
     102,    0,  590,    2, 0x08 /* Private */,
     103,    0,  591,    2, 0x08 /* Private */,
     104,    0,  592,    2, 0x08 /* Private */,
     105,    1,  593,    2, 0x08 /* Private */,
     106,    0,  596,    2, 0x08 /* Private */,
     107,    0,  597,    2, 0x08 /* Private */,
     108,    0,  598,    2, 0x08 /* Private */,
     109,    0,  599,    2, 0x08 /* Private */,
     110,    0,  600,    2, 0x08 /* Private */,
     111,    1,  601,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QString,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   23,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   23,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::UShort, QMetaType::Bool,   34,   35,   36,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UShort,   35,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   43,
    QMetaType::Void, QMetaType::QString,   45,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   48,
    QMetaType::Void, QMetaType::QString,   48,
    QMetaType::Void, QMetaType::QString,   48,
    QMetaType::Void, QMetaType::QString,   48,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, QMetaType::QString, QMetaType::QString,   55,   56,
    QMetaType::Void, QMetaType::QString,   58,
    QMetaType::Void, 0x80000000 | 60,   58,
    QMetaType::Void, 0x80000000 | 60,   58,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 64,   65,
    QMetaType::Void, QMetaType::Bool,   67,
    QMetaType::Void, QMetaType::QString,   69,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 72, QMetaType::QByteArray,   73,   74,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   79,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   79,   81,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString,   83,   84,   81,
    QMetaType::Void, QMetaType::Bool,   86,
    QMetaType::Void, QMetaType::Bool,   86,
    QMetaType::Void, QMetaType::Bool,   86,
    QMetaType::Void, QMetaType::Bool,   90,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  101,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   69,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QByteArray,    2,

       0        // eod
};

void HWForm::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HWForm *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->GoToSaves(); break;
        case 1: _t->GoToDemos(); break;
        case 2: _t->GoToNet(); break;
        case 3: _t->GoToHelp(); break;
        case 4: _t->GoToEditWeapons(); break;
        case 5: _t->GoToNewWeapons(); break;
        case 6: _t->GoToWeapons((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->GoToScheme((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->GoToEditScheme(); break;
        case 9: _t->GoToNewScheme(); break;
        case 10: _t->GoToPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->GoBack(); break;
        case 12: _t->OpenSnapshotFolder(); break;
        case 13: { QString _r = _t->getDemoArguments();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->AssociateFiles(); break;
        case 15: _t->btnExitPressed(); break;
        case 16: _t->IntermediateSetup(); break;
        case 17: _t->NewTeam(); break;
        case 18: _t->EditTeam((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 19: _t->AfterTeamEdit(); break;
        case 20: _t->DeleteTeam((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 21: _t->DeleteScheme(); break;
        case 22: _t->DeleteWeaponSet(); break;
        case 23: _t->SimpleGame(); break;
        case 24: _t->PlayDemo(); break;
        case 25: _t->startTraining((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 26: _t->StartCampaign(); break;
        case 27: _t->NetConnect(); break;
        case 28: _t->NetConnectServer((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< quint16(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 29: _t->NetConnectOfficialServer(); break;
        case 30: _t->NetStartServer(); break;
        case 31: _t->NetRedirected((*reinterpret_cast< quint16(*)>(_a[1]))); break;
        case 32: _t->NetDisconnect(); break;
        case 33: _t->NetConnected(); break;
        case 34: _t->NetError((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 35: _t->NetWarning((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 36: _t->NetGameEnter(); break;
        case 37: _t->NetPassword((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 38: _t->NetNickRegistered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 39: _t->NetNickNotRegistered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 40: _t->NetNickTaken((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 41: _t->NetAuthFailed(); break;
        case 42: _t->askRoomPassword(); break;
        case 43: { bool _r = _t->RetryDialog((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 44: _t->NetTeamAccepted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 45: _t->AddNetTeam((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 46: _t->RemoveNetTeam((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 47: _t->StartMPGame(); break;
        case 48: _t->GameStateChanged((*reinterpret_cast< GameState(*)>(_a[1]))); break;
        case 49: _t->DemoPresenceChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 50: _t->ForcedDisconnect((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 51: _t->ShowFatalErrorMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 52: _t->GetRecord((*reinterpret_cast< RecordType(*)>(_a[1])),(*reinterpret_cast< const QByteArray(*)>(_a[2]))); break;
        case 53: _t->CreateNetGame(); break;
        case 54: _t->PlayOfficialServerDemo(); break;
        case 55: _t->UpdateWeapons(); break;
        case 56: _t->DeleteWeapons((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 57: _t->AddWeapons((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 58: _t->EditWeapons((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< QString(*)>(_a[3]))); break;
        case 59: _t->onFrontendFullscreen((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 60: _t->onFrontendEffects((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 61: _t->onFrontendSoundsToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 62: _t->Music((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 63: _t->UpdateCampaignPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 64: _t->UpdateCampaignPageTeam((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 65: _t->UpdateCampaignPageProgress((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 66: _t->UpdateCampaignPageMission((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 67: _t->UpdateTrainingPageTeam((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 68: _t->InitCampaignPage(); break;
        case 69: _t->RestoreSingleplayerTeamSelection(); break;
        case 70: _t->showFeedbackDialog(); break;
        case 71: _t->showFeedbackDialogNetChecked(); break;
        case 72: _t->NetGameChangeStatus((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 73: _t->NetGameMaster(); break;
        case 74: _t->NetGameSlave(); break;
        case 75: _t->AsyncNetServerStart(); break;
        case 76: _t->NetLeftRoom((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 77: _t->selectFirstNetScheme(); break;
        case 78: _t->saveDemoWithCustomName(); break;
        case 79: _t->openRegistrationPage(); break;
        case 80: _t->startGame(); break;
        case 81: _t->restartGame(); break;
        case 82: _t->FromNetProxySlot((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject HWForm::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_HWForm.data,
    qt_meta_data_HWForm,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HWForm::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HWForm::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HWForm.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int HWForm::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 83)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 83;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 83)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 83;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
