/****************************************************************************
** Meta object code from reading C++ file 'pageroomslist.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/page/pageroomslist.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pageroomslist.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PageRoomsList_t {
    QByteArrayData data[26];
    char stringdata0[355];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PageRoomsList_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PageRoomsList_t qt_meta_stringdata_PageRoomsList = {
    {
QT_MOC_LITERAL(0, 0, 13), // "PageRoomsList"
QT_MOC_LITERAL(1, 14, 16), // "askForCreateRoom"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 14), // "askForJoinRoom"
QT_MOC_LITERAL(4, 47, 14), // "askForRoomList"
QT_MOC_LITERAL(5, 62, 19), // "askJoinConfirmation"
QT_MOC_LITERAL(6, 82, 8), // "setAdmin"
QT_MOC_LITERAL(7, 91, 7), // "setUser"
QT_MOC_LITERAL(8, 99, 8), // "nickname"
QT_MOC_LITERAL(9, 108, 17), // "updateNickCounter"
QT_MOC_LITERAL(10, 126, 3), // "cnt"
QT_MOC_LITERAL(11, 130, 13), // "onCreateClick"
QT_MOC_LITERAL(12, 144, 11), // "onJoinClick"
QT_MOC_LITERAL(13, 156, 14), // "onRefreshClick"
QT_MOC_LITERAL(14, 171, 18), // "onJoinConfirmation"
QT_MOC_LITERAL(15, 190, 22), // "onSortIndicatorChanged"
QT_MOC_LITERAL(16, 213, 12), // "logicalIndex"
QT_MOC_LITERAL(17, 226, 13), // "Qt::SortOrder"
QT_MOC_LITERAL(18, 240, 5), // "order"
QT_MOC_LITERAL(19, 246, 15), // "onFilterChanged"
QT_MOC_LITERAL(20, 262, 16), // "onRoomNameChosen"
QT_MOC_LITERAL(21, 279, 8), // "password"
QT_MOC_LITERAL(22, 288, 20), // "roomSelectionChanged"
QT_MOC_LITERAL(23, 309, 11), // "QModelIndex"
QT_MOC_LITERAL(24, 321, 15), // "moveSelectionUp"
QT_MOC_LITERAL(25, 337, 17) // "moveSelectionDown"

    },
    "PageRoomsList\0askForCreateRoom\0\0"
    "askForJoinRoom\0askForRoomList\0"
    "askJoinConfirmation\0setAdmin\0setUser\0"
    "nickname\0updateNickCounter\0cnt\0"
    "onCreateClick\0onJoinClick\0onRefreshClick\0"
    "onJoinConfirmation\0onSortIndicatorChanged\0"
    "logicalIndex\0Qt::SortOrder\0order\0"
    "onFilterChanged\0onRoomNameChosen\0"
    "password\0roomSelectionChanged\0QModelIndex\0"
    "moveSelectionUp\0moveSelectionDown"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PageRoomsList[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,   99,    2, 0x06 /* Public */,
       3,    2,  104,    2, 0x06 /* Public */,
       4,    0,  109,    2, 0x06 /* Public */,
       5,    1,  110,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    1,  113,    2, 0x0a /* Public */,
       7,    1,  116,    2, 0x0a /* Public */,
       9,    1,  119,    2, 0x0a /* Public */,
      11,    0,  122,    2, 0x08 /* Private */,
      12,    0,  123,    2, 0x08 /* Private */,
      13,    0,  124,    2, 0x08 /* Private */,
      14,    1,  125,    2, 0x08 /* Private */,
      15,    2,  128,    2, 0x08 /* Private */,
      19,    0,  133,    2, 0x08 /* Private */,
      20,    2,  134,    2, 0x08 /* Private */,
      22,    2,  139,    2, 0x08 /* Private */,
      24,    0,  144,    2, 0x08 /* Private */,
      25,    0,  145,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    2,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    2,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 17,   16,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    2,   21,
    QMetaType::Void, 0x80000000 | 23, 0x80000000 | 23,    2,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PageRoomsList::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageRoomsList *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->askForCreateRoom((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 1: _t->askForJoinRoom((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 2: _t->askForRoomList(); break;
        case 3: _t->askJoinConfirmation((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->setAdmin((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->setUser((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->updateNickCounter((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->onCreateClick(); break;
        case 8: _t->onJoinClick(); break;
        case 9: _t->onRefreshClick(); break;
        case 10: _t->onJoinConfirmation((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->onSortIndicatorChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Qt::SortOrder(*)>(_a[2]))); break;
        case 12: _t->onFilterChanged(); break;
        case 13: _t->onRoomNameChosen((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 14: _t->roomSelectionChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 15: _t->moveSelectionUp(); break;
        case 16: _t->moveSelectionDown(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PageRoomsList::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageRoomsList::askForCreateRoom)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PageRoomsList::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageRoomsList::askForJoinRoom)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PageRoomsList::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageRoomsList::askForRoomList)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (PageRoomsList::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageRoomsList::askJoinConfirmation)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PageRoomsList::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPage::staticMetaObject>(),
    qt_meta_stringdata_PageRoomsList.data,
    qt_meta_data_PageRoomsList,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PageRoomsList::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageRoomsList::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PageRoomsList.stringdata0))
        return static_cast<void*>(this);
    return AbstractPage::qt_metacast(_clname);
}

int PageRoomsList::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void PageRoomsList::askForCreateRoom(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PageRoomsList::askForJoinRoom(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PageRoomsList::askForRoomList()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void PageRoomsList::askJoinConfirmation(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
