/****************************************************************************
** Meta object code from reading C++ file 'newnetclient.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../QTfrontend/net/newnetclient.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'newnetclient.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_HWNewNet_t {
    QByteArrayData data[108];
    char stringdata0[1328];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HWNewNet_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HWNewNet_t qt_meta_stringdata_HWNewNet = {
    {
QT_MOC_LITERAL(0, 0, 8), // "HWNewNet"
QT_MOC_LITERAL(1, 9, 13), // "AskForRunGame"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 24), // "AskForOfficialServerDemo"
QT_MOC_LITERAL(4, 49, 9), // "connected"
QT_MOC_LITERAL(5, 59, 12), // "disconnected"
QT_MOC_LITERAL(6, 72, 6), // "reason"
QT_MOC_LITERAL(7, 79, 10), // "redirected"
QT_MOC_LITERAL(8, 90, 4), // "port"
QT_MOC_LITERAL(9, 95, 5), // "Error"
QT_MOC_LITERAL(10, 101, 6), // "errmsg"
QT_MOC_LITERAL(11, 108, 7), // "Warning"
QT_MOC_LITERAL(12, 116, 6), // "wrnmsg"
QT_MOC_LITERAL(13, 123, 14), // "NickRegistered"
QT_MOC_LITERAL(14, 138, 4), // "nick"
QT_MOC_LITERAL(15, 143, 17), // "NickNotRegistered"
QT_MOC_LITERAL(16, 161, 9), // "NickTaken"
QT_MOC_LITERAL(17, 171, 10), // "AuthFailed"
QT_MOC_LITERAL(18, 182, 11), // "EnteredGame"
QT_MOC_LITERAL(19, 194, 8), // "LeftRoom"
QT_MOC_LITERAL(20, 203, 7), // "FromNet"
QT_MOC_LITERAL(21, 211, 3), // "buf"
QT_MOC_LITERAL(22, 215, 11), // "adminAccess"
QT_MOC_LITERAL(23, 227, 10), // "roomMaster"
QT_MOC_LITERAL(24, 238, 15), // "roomNameUpdated"
QT_MOC_LITERAL(25, 254, 4), // "name"
QT_MOC_LITERAL(26, 259, 18), // "askForRoomPassword"
QT_MOC_LITERAL(27, 278, 15), // "netSchemeConfig"
QT_MOC_LITERAL(28, 294, 12), // "paramChanged"
QT_MOC_LITERAL(29, 307, 5), // "param"
QT_MOC_LITERAL(30, 313, 5), // "value"
QT_MOC_LITERAL(31, 319, 11), // "configAsked"
QT_MOC_LITERAL(32, 331, 12), // "TeamAccepted"
QT_MOC_LITERAL(33, 344, 10), // "AddNetTeam"
QT_MOC_LITERAL(34, 355, 6), // "HWTeam"
QT_MOC_LITERAL(35, 362, 13), // "RemoveNetTeam"
QT_MOC_LITERAL(36, 376, 12), // "hhnumChanged"
QT_MOC_LITERAL(37, 389, 16), // "teamColorChanged"
QT_MOC_LITERAL(38, 406, 10), // "playerInfo"
QT_MOC_LITERAL(39, 417, 2), // "ip"
QT_MOC_LITERAL(40, 420, 7), // "version"
QT_MOC_LITERAL(41, 428, 8), // "roomInfo"
QT_MOC_LITERAL(42, 437, 16), // "lobbyChatMessage"
QT_MOC_LITERAL(43, 454, 7), // "message"
QT_MOC_LITERAL(44, 462, 15), // "lobbyChatAction"
QT_MOC_LITERAL(45, 478, 6), // "action"
QT_MOC_LITERAL(46, 485, 15), // "roomChatMessage"
QT_MOC_LITERAL(47, 501, 14), // "roomChatAction"
QT_MOC_LITERAL(48, 516, 17), // "chatStringFromNet"
QT_MOC_LITERAL(49, 534, 9), // "roomsList"
QT_MOC_LITERAL(50, 544, 13), // "serverMessage"
QT_MOC_LITERAL(51, 558, 16), // "serverMessageNew"
QT_MOC_LITERAL(52, 575, 16), // "serverMessageOld"
QT_MOC_LITERAL(53, 592, 17), // "latestProtocolVar"
QT_MOC_LITERAL(54, 610, 8), // "bansList"
QT_MOC_LITERAL(55, 619, 16), // "setMyReadyStatus"
QT_MOC_LITERAL(56, 636, 7), // "isReady"
QT_MOC_LITERAL(57, 644, 16), // "messageProcessed"
QT_MOC_LITERAL(58, 661, 11), // "ToggleReady"
QT_MOC_LITERAL(59, 673, 13), // "chatLineToNet"
QT_MOC_LITERAL(60, 687, 3), // "str"
QT_MOC_LITERAL(61, 691, 21), // "chatLineToNetWithEcho"
QT_MOC_LITERAL(62, 713, 15), // "chatLineToLobby"
QT_MOC_LITERAL(63, 729, 15), // "SendTeamMessage"
QT_MOC_LITERAL(64, 745, 7), // "SendNet"
QT_MOC_LITERAL(65, 753, 7), // "AddTeam"
QT_MOC_LITERAL(66, 761, 4), // "team"
QT_MOC_LITERAL(67, 766, 10), // "RemoveTeam"
QT_MOC_LITERAL(68, 777, 21), // "onHedgehogsNumChanged"
QT_MOC_LITERAL(69, 799, 18), // "onTeamColorChanged"
QT_MOC_LITERAL(70, 818, 14), // "onParamChanged"
QT_MOC_LITERAL(71, 833, 19), // "setServerMessageNew"
QT_MOC_LITERAL(72, 853, 19), // "setServerMessageOld"
QT_MOC_LITERAL(73, 873, 20), // "setLatestProtocolVar"
QT_MOC_LITERAL(74, 894, 5), // "proto"
QT_MOC_LITERAL(75, 900, 13), // "askServerVars"
QT_MOC_LITERAL(76, 914, 8), // "JoinRoom"
QT_MOC_LITERAL(77, 923, 4), // "room"
QT_MOC_LITERAL(78, 928, 8), // "password"
QT_MOC_LITERAL(79, 937, 10), // "CreateRoom"
QT_MOC_LITERAL(80, 948, 14), // "updateRoomName"
QT_MOC_LITERAL(81, 963, 12), // "askRoomsList"
QT_MOC_LITERAL(82, 976, 12), // "gameFinished"
QT_MOC_LITERAL(83, 989, 8), // "correcly"
QT_MOC_LITERAL(84, 998, 9), // "banPlayer"
QT_MOC_LITERAL(85, 1008, 10), // "kickPlayer"
QT_MOC_LITERAL(86, 1019, 16), // "delegateToPlayer"
QT_MOC_LITERAL(87, 1036, 10), // "infoPlayer"
QT_MOC_LITERAL(88, 1047, 12), // "followPlayer"
QT_MOC_LITERAL(89, 1060, 14), // "consoleCommand"
QT_MOC_LITERAL(90, 1075, 9), // "startGame"
QT_MOC_LITERAL(91, 1085, 19), // "toggleRestrictJoins"
QT_MOC_LITERAL(92, 1105, 22), // "toggleRestrictTeamAdds"
QT_MOC_LITERAL(93, 1128, 20), // "toggleRegisteredOnly"
QT_MOC_LITERAL(94, 1149, 8), // "partRoom"
QT_MOC_LITERAL(95, 1158, 18), // "clearAccountsCache"
QT_MOC_LITERAL(96, 1177, 10), // "getBanList"
QT_MOC_LITERAL(97, 1188, 9), // "removeBan"
QT_MOC_LITERAL(98, 1198, 5), // "banIP"
QT_MOC_LITERAL(99, 1204, 7), // "seconds"
QT_MOC_LITERAL(100, 1212, 7), // "banNick"
QT_MOC_LITERAL(101, 1220, 19), // "roomPasswordEntered"
QT_MOC_LITERAL(102, 1240, 10), // "ClientRead"
QT_MOC_LITERAL(103, 1251, 9), // "OnConnect"
QT_MOC_LITERAL(104, 1261, 12), // "OnDisconnect"
QT_MOC_LITERAL(105, 1274, 12), // "displayError"
QT_MOC_LITERAL(106, 1287, 28), // "QAbstractSocket::SocketError"
QT_MOC_LITERAL(107, 1316, 11) // "socketError"

    },
    "HWNewNet\0AskForRunGame\0\0"
    "AskForOfficialServerDemo\0connected\0"
    "disconnected\0reason\0redirected\0port\0"
    "Error\0errmsg\0Warning\0wrnmsg\0NickRegistered\0"
    "nick\0NickNotRegistered\0NickTaken\0"
    "AuthFailed\0EnteredGame\0LeftRoom\0FromNet\0"
    "buf\0adminAccess\0roomMaster\0roomNameUpdated\0"
    "name\0askForRoomPassword\0netSchemeConfig\0"
    "paramChanged\0param\0value\0configAsked\0"
    "TeamAccepted\0AddNetTeam\0HWTeam\0"
    "RemoveNetTeam\0hhnumChanged\0teamColorChanged\0"
    "playerInfo\0ip\0version\0roomInfo\0"
    "lobbyChatMessage\0message\0lobbyChatAction\0"
    "action\0roomChatMessage\0roomChatAction\0"
    "chatStringFromNet\0roomsList\0serverMessage\0"
    "serverMessageNew\0serverMessageOld\0"
    "latestProtocolVar\0bansList\0setMyReadyStatus\0"
    "isReady\0messageProcessed\0ToggleReady\0"
    "chatLineToNet\0str\0chatLineToNetWithEcho\0"
    "chatLineToLobby\0SendTeamMessage\0SendNet\0"
    "AddTeam\0team\0RemoveTeam\0onHedgehogsNumChanged\0"
    "onTeamColorChanged\0onParamChanged\0"
    "setServerMessageNew\0setServerMessageOld\0"
    "setLatestProtocolVar\0proto\0askServerVars\0"
    "JoinRoom\0room\0password\0CreateRoom\0"
    "updateRoomName\0askRoomsList\0gameFinished\0"
    "correcly\0banPlayer\0kickPlayer\0"
    "delegateToPlayer\0infoPlayer\0followPlayer\0"
    "consoleCommand\0startGame\0toggleRestrictJoins\0"
    "toggleRestrictTeamAdds\0toggleRegisteredOnly\0"
    "partRoom\0clearAccountsCache\0getBanList\0"
    "removeBan\0banIP\0seconds\0banNick\0"
    "roomPasswordEntered\0ClientRead\0OnConnect\0"
    "OnDisconnect\0displayError\0"
    "QAbstractSocket::SocketError\0socketError"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HWNewNet[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      81,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      40,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  419,    2, 0x06 /* Public */,
       3,    0,  420,    2, 0x06 /* Public */,
       4,    0,  421,    2, 0x06 /* Public */,
       5,    1,  422,    2, 0x06 /* Public */,
       7,    1,  425,    2, 0x06 /* Public */,
       9,    1,  428,    2, 0x06 /* Public */,
      11,    1,  431,    2, 0x06 /* Public */,
      13,    1,  434,    2, 0x06 /* Public */,
      15,    1,  437,    2, 0x06 /* Public */,
      16,    1,  440,    2, 0x06 /* Public */,
      17,    0,  443,    2, 0x06 /* Public */,
      18,    0,  444,    2, 0x06 /* Public */,
      19,    1,  445,    2, 0x06 /* Public */,
      20,    1,  448,    2, 0x06 /* Public */,
      22,    1,  451,    2, 0x06 /* Public */,
      23,    1,  454,    2, 0x06 /* Public */,
      24,    1,  457,    2, 0x06 /* Public */,
      26,    0,  460,    2, 0x06 /* Public */,
      27,    1,  461,    2, 0x06 /* Public */,
      28,    2,  464,    2, 0x06 /* Public */,
      31,    0,  469,    2, 0x06 /* Public */,
      32,    1,  470,    2, 0x06 /* Public */,
      33,    1,  473,    2, 0x06 /* Public */,
      35,    1,  476,    2, 0x06 /* Public */,
      36,    1,  479,    2, 0x06 /* Public */,
      37,    1,  482,    2, 0x06 /* Public */,
      38,    4,  485,    2, 0x06 /* Public */,
      42,    2,  494,    2, 0x06 /* Public */,
      44,    2,  499,    2, 0x06 /* Public */,
      46,    2,  504,    2, 0x06 /* Public */,
      47,    2,  509,    2, 0x06 /* Public */,
      48,    1,  514,    2, 0x06 /* Public */,
      49,    1,  517,    2, 0x06 /* Public */,
      50,    1,  520,    2, 0x06 /* Public */,
      51,    1,  523,    2, 0x06 /* Public */,
      52,    1,  526,    2, 0x06 /* Public */,
      53,    1,  529,    2, 0x06 /* Public */,
      54,    1,  532,    2, 0x06 /* Public */,
      55,    1,  535,    2, 0x06 /* Public */,
      57,    0,  538,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      58,    0,  539,    2, 0x0a /* Public */,
      59,    1,  540,    2, 0x0a /* Public */,
      61,    1,  543,    2, 0x0a /* Public */,
      62,    1,  546,    2, 0x0a /* Public */,
      63,    1,  549,    2, 0x0a /* Public */,
      64,    1,  552,    2, 0x0a /* Public */,
      65,    1,  555,    2, 0x0a /* Public */,
      67,    1,  558,    2, 0x0a /* Public */,
      68,    1,  561,    2, 0x0a /* Public */,
      69,    1,  564,    2, 0x0a /* Public */,
      70,    2,  567,    2, 0x0a /* Public */,
      71,    1,  572,    2, 0x0a /* Public */,
      72,    1,  575,    2, 0x0a /* Public */,
      73,    1,  578,    2, 0x0a /* Public */,
      75,    0,  581,    2, 0x0a /* Public */,
      76,    2,  582,    2, 0x0a /* Public */,
      79,    2,  587,    2, 0x0a /* Public */,
      80,    1,  592,    2, 0x0a /* Public */,
      81,    0,  595,    2, 0x0a /* Public */,
      82,    1,  596,    2, 0x0a /* Public */,
      84,    1,  599,    2, 0x0a /* Public */,
      85,    1,  602,    2, 0x0a /* Public */,
      86,    1,  605,    2, 0x0a /* Public */,
      87,    1,  608,    2, 0x0a /* Public */,
      88,    1,  611,    2, 0x0a /* Public */,
      89,    1,  614,    2, 0x0a /* Public */,
      90,    0,  617,    2, 0x0a /* Public */,
      91,    0,  618,    2, 0x0a /* Public */,
      92,    0,  619,    2, 0x0a /* Public */,
      93,    0,  620,    2, 0x0a /* Public */,
      94,    0,  621,    2, 0x0a /* Public */,
      95,    0,  622,    2, 0x0a /* Public */,
      96,    0,  623,    2, 0x0a /* Public */,
      97,    1,  624,    2, 0x0a /* Public */,
      98,    3,  627,    2, 0x0a /* Public */,
     100,    3,  634,    2, 0x0a /* Public */,
     101,    1,  641,    2, 0x0a /* Public */,
     102,    0,  644,    2, 0x08 /* Private */,
     103,    0,  645,    2, 0x08 /* Private */,
     104,    0,  646,    2, 0x08 /* Private */,
     105,    1,  647,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::UShort,    8,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QByteArray,   21,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,   25,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QStringList,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QStringList,   29,   30,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 34,    2,
    QMetaType::Void, 0x80000000 | 34,    2,
    QMetaType::Void, 0x80000000 | 34,    2,
    QMetaType::Void, 0x80000000 | 34,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   14,   39,   40,   41,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   14,   43,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   14,   45,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   14,   43,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   14,   45,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QStringList,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QStringList,    2,
    QMetaType::Void, QMetaType::Bool,   56,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   60,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,   60,
    QMetaType::Void, QMetaType::QString,   60,
    QMetaType::Void, QMetaType::QByteArray,   21,
    QMetaType::Void, 0x80000000 | 34,   66,
    QMetaType::Void, 0x80000000 | 34,   66,
    QMetaType::Void, 0x80000000 | 34,   66,
    QMetaType::Void, 0x80000000 | 34,   66,
    QMetaType::Void, QMetaType::QString, QMetaType::QStringList,   29,   30,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,   74,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   77,   78,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   77,   78,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   83,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int,   39,    6,   99,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int,   14,    6,   99,
    QMetaType::Void, QMetaType::QString,   78,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 106,  107,

       0        // eod
};

void HWNewNet::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HWNewNet *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->AskForRunGame(); break;
        case 1: _t->AskForOfficialServerDemo(); break;
        case 2: _t->connected(); break;
        case 3: _t->disconnected((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->redirected((*reinterpret_cast< quint16(*)>(_a[1]))); break;
        case 5: _t->Error((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->Warning((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->NickRegistered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->NickNotRegistered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->NickTaken((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->AuthFailed(); break;
        case 11: _t->EnteredGame(); break;
        case 12: _t->LeftRoom((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->FromNet((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 14: _t->adminAccess((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->roomMaster((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->roomNameUpdated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->askForRoomPassword(); break;
        case 18: _t->netSchemeConfig((*reinterpret_cast< QStringList(*)>(_a[1]))); break;
        case 19: _t->paramChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        case 20: _t->configAsked(); break;
        case 21: _t->TeamAccepted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 22: _t->AddNetTeam((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 23: _t->RemoveNetTeam((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 24: _t->hhnumChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 25: _t->teamColorChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 26: _t->playerInfo((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 27: _t->lobbyChatMessage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 28: _t->lobbyChatAction((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 29: _t->roomChatMessage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 30: _t->roomChatAction((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 31: _t->chatStringFromNet((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 32: _t->roomsList((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 33: _t->serverMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 34: _t->serverMessageNew((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 35: _t->serverMessageOld((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 36: _t->latestProtocolVar((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 37: _t->bansList((*reinterpret_cast< const QStringList(*)>(_a[1]))); break;
        case 38: _t->setMyReadyStatus((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 39: _t->messageProcessed(); break;
        case 40: _t->ToggleReady(); break;
        case 41: _t->chatLineToNet((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 42: _t->chatLineToNetWithEcho((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 43: _t->chatLineToLobby((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 44: _t->SendTeamMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 45: _t->SendNet((*reinterpret_cast< const QByteArray(*)>(_a[1]))); break;
        case 46: _t->AddTeam((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 47: _t->RemoveTeam((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 48: _t->onHedgehogsNumChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 49: _t->onTeamColorChanged((*reinterpret_cast< const HWTeam(*)>(_a[1]))); break;
        case 50: _t->onParamChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QStringList(*)>(_a[2]))); break;
        case 51: _t->setServerMessageNew((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 52: _t->setServerMessageOld((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 53: _t->setLatestProtocolVar((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 54: _t->askServerVars(); break;
        case 55: _t->JoinRoom((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 56: _t->CreateRoom((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 57: _t->updateRoomName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 58: _t->askRoomsList(); break;
        case 59: _t->gameFinished((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 60: _t->banPlayer((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 61: _t->kickPlayer((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 62: _t->delegateToPlayer((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 63: _t->infoPlayer((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 64: _t->followPlayer((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 65: _t->consoleCommand((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 66: _t->startGame(); break;
        case 67: _t->toggleRestrictJoins(); break;
        case 68: _t->toggleRestrictTeamAdds(); break;
        case 69: _t->toggleRegisteredOnly(); break;
        case 70: _t->partRoom(); break;
        case 71: _t->clearAccountsCache(); break;
        case 72: _t->getBanList(); break;
        case 73: _t->removeBan((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 74: _t->banIP((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 75: _t->banNick((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 76: _t->roomPasswordEntered((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 77: _t->ClientRead(); break;
        case 78: _t->OnConnect(); break;
        case 79: _t->OnDisconnect(); break;
        case 80: _t->displayError((*reinterpret_cast< QAbstractSocket::SocketError(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HWNewNet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::AskForRunGame)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::AskForOfficialServerDemo)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::connected)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::disconnected)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(quint16 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::redirected)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::Error)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::Warning)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::NickRegistered)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::NickNotRegistered)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::NickTaken)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::AuthFailed)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::EnteredGame)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::LeftRoom)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QByteArray & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::FromNet)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::adminAccess)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::roomMaster)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::roomNameUpdated)) {
                *result = 16;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::askForRoomPassword)) {
                *result = 17;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(QStringList );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::netSchemeConfig)) {
                *result = 18;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & , const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::paramChanged)) {
                *result = 19;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::configAsked)) {
                *result = 20;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::TeamAccepted)) {
                *result = 21;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::AddNetTeam)) {
                *result = 22;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::RemoveNetTeam)) {
                *result = 23;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::hhnumChanged)) {
                *result = 24;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const HWTeam & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::teamColorChanged)) {
                *result = 25;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & , const QString & , const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::playerInfo)) {
                *result = 26;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::lobbyChatMessage)) {
                *result = 27;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::lobbyChatAction)) {
                *result = 28;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::roomChatMessage)) {
                *result = 29;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & , const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::roomChatAction)) {
                *result = 30;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::chatStringFromNet)) {
                *result = 31;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::roomsList)) {
                *result = 32;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::serverMessage)) {
                *result = 33;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::serverMessageNew)) {
                *result = 34;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::serverMessageOld)) {
                *result = 35;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::latestProtocolVar)) {
                *result = 36;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(const QStringList & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::bansList)) {
                *result = 37;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::setMyReadyStatus)) {
                *result = 38;
                return;
            }
        }
        {
            using _t = void (HWNewNet::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWNewNet::messageProcessed)) {
                *result = 39;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject HWNewNet::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_HWNewNet.data,
    qt_meta_data_HWNewNet,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HWNewNet::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HWNewNet::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HWNewNet.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int HWNewNet::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 81)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 81;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 81)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 81;
    }
    return _id;
}

// SIGNAL 0
void HWNewNet::AskForRunGame()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void HWNewNet::AskForOfficialServerDemo()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void HWNewNet::connected()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void HWNewNet::disconnected(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void HWNewNet::redirected(quint16 _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void HWNewNet::Error(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void HWNewNet::Warning(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void HWNewNet::NickRegistered(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void HWNewNet::NickNotRegistered(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void HWNewNet::NickTaken(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void HWNewNet::AuthFailed()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}

// SIGNAL 11
void HWNewNet::EnteredGame()
{
    QMetaObject::activate(this, &staticMetaObject, 11, nullptr);
}

// SIGNAL 12
void HWNewNet::LeftRoom(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void HWNewNet::FromNet(const QByteArray & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void HWNewNet::adminAccess(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void HWNewNet::roomMaster(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void HWNewNet::roomNameUpdated(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}

// SIGNAL 17
void HWNewNet::askForRoomPassword()
{
    QMetaObject::activate(this, &staticMetaObject, 17, nullptr);
}

// SIGNAL 18
void HWNewNet::netSchemeConfig(QStringList _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 18, _a);
}

// SIGNAL 19
void HWNewNet::paramChanged(const QString & _t1, const QStringList & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 19, _a);
}

// SIGNAL 20
void HWNewNet::configAsked()
{
    QMetaObject::activate(this, &staticMetaObject, 20, nullptr);
}

// SIGNAL 21
void HWNewNet::TeamAccepted(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 21, _a);
}

// SIGNAL 22
void HWNewNet::AddNetTeam(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 22, _a);
}

// SIGNAL 23
void HWNewNet::RemoveNetTeam(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 23, _a);
}

// SIGNAL 24
void HWNewNet::hhnumChanged(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 24, _a);
}

// SIGNAL 25
void HWNewNet::teamColorChanged(const HWTeam & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 25, _a);
}

// SIGNAL 26
void HWNewNet::playerInfo(const QString & _t1, const QString & _t2, const QString & _t3, const QString & _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 26, _a);
}

// SIGNAL 27
void HWNewNet::lobbyChatMessage(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 27, _a);
}

// SIGNAL 28
void HWNewNet::lobbyChatAction(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 28, _a);
}

// SIGNAL 29
void HWNewNet::roomChatMessage(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 29, _a);
}

// SIGNAL 30
void HWNewNet::roomChatAction(const QString & _t1, const QString & _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 30, _a);
}

// SIGNAL 31
void HWNewNet::chatStringFromNet(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 31, _a);
}

// SIGNAL 32
void HWNewNet::roomsList(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 32, _a);
}

// SIGNAL 33
void HWNewNet::serverMessage(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 33, _a);
}

// SIGNAL 34
void HWNewNet::serverMessageNew(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 34, _a);
}

// SIGNAL 35
void HWNewNet::serverMessageOld(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 35, _a);
}

// SIGNAL 36
void HWNewNet::latestProtocolVar(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 36, _a);
}

// SIGNAL 37
void HWNewNet::bansList(const QStringList & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 37, _a);
}

// SIGNAL 38
void HWNewNet::setMyReadyStatus(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 38, _a);
}

// SIGNAL 39
void HWNewNet::messageProcessed()
{
    QMetaObject::activate(this, &staticMetaObject, 39, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
