/****************************************************************************
** Meta object code from reading C++ file 'pagevideos.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/page/pagevideos.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pagevideos.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PageVideos_t {
    QByteArrayData data[18];
    char stringdata0[221];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PageVideos_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PageVideos_t qt_meta_stringdata_PageVideos = {
    {
QT_MOC_LITERAL(0, 0, 10), // "PageVideos"
QT_MOC_LITERAL(1, 11, 16), // "encodingFinished"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 7), // "success"
QT_MOC_LITERAL(4, 37, 14), // "updateProgress"
QT_MOC_LITERAL(5, 52, 5), // "value"
QT_MOC_LITERAL(6, 58, 17), // "cellDoubleClicked"
QT_MOC_LITERAL(7, 76, 3), // "row"
QT_MOC_LITERAL(8, 80, 6), // "column"
QT_MOC_LITERAL(9, 87, 11), // "cellChanged"
QT_MOC_LITERAL(10, 99, 18), // "currentCellChanged"
QT_MOC_LITERAL(11, 118, 16), // "playSelectedFile"
QT_MOC_LITERAL(12, 135, 19), // "deleteSelectedFiles"
QT_MOC_LITERAL(13, 155, 19), // "openVideosDirectory"
QT_MOC_LITERAL(14, 175, 14), // "updateFileList"
QT_MOC_LITERAL(15, 190, 4), // "path"
QT_MOC_LITERAL(16, 195, 21), // "ShowFatalErrorMessage"
QT_MOC_LITERAL(17, 217, 3) // "msg"

    },
    "PageVideos\0encodingFinished\0\0success\0"
    "updateProgress\0value\0cellDoubleClicked\0"
    "row\0column\0cellChanged\0currentCellChanged\0"
    "playSelectedFile\0deleteSelectedFiles\0"
    "openVideosDirectory\0updateFileList\0"
    "path\0ShowFatalErrorMessage\0msg"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PageVideos[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x08 /* Private */,
       4,    1,   67,    2, 0x08 /* Private */,
       6,    2,   70,    2, 0x08 /* Private */,
       9,    2,   75,    2, 0x08 /* Private */,
      10,    0,   80,    2, 0x08 /* Private */,
      11,    0,   81,    2, 0x08 /* Private */,
      12,    0,   82,    2, 0x08 /* Private */,
      13,    0,   83,    2, 0x08 /* Private */,
      14,    1,   84,    2, 0x08 /* Private */,
      16,    1,   87,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Float,    5,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    7,    8,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    7,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Void, QMetaType::QString,   17,

       0        // eod
};

void PageVideos::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageVideos *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->encodingFinished((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->updateProgress((*reinterpret_cast< float(*)>(_a[1]))); break;
        case 2: _t->cellDoubleClicked((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 3: _t->cellChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->currentCellChanged(); break;
        case 5: _t->playSelectedFile(); break;
        case 6: _t->deleteSelectedFiles(); break;
        case 7: _t->openVideosDirectory(); break;
        case 8: _t->updateFileList((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->ShowFatalErrorMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PageVideos::staticMetaObject = { {
    QMetaObject::SuperData::link<AbstractPage::staticMetaObject>(),
    qt_meta_stringdata_PageVideos.data,
    qt_meta_data_PageVideos,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PageVideos::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageVideos::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PageVideos.stringdata0))
        return static_cast<void*>(this);
    return AbstractPage::qt_metacast(_clname);
}

int PageVideos::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = AbstractPage::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
