/****************************************************************************
** Meta object code from reading C++ file 'tcpBase.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../QTfrontend/net/tcpBase.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tcpBase.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TCPBase_t {
    QByteArrayData data[14];
    char stringdata0[179];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TCPBase_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TCPBase_t qt_meta_stringdata_TCPBase = {
    {
QT_MOC_LITERAL(0, 0, 7), // "TCPBase"
QT_MOC_LITERAL(1, 8, 10), // "isReadyNow"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 13), // "NewConnection"
QT_MOC_LITERAL(4, 34, 16), // "ClientDisconnect"
QT_MOC_LITERAL(5, 51, 10), // "ClientRead"
QT_MOC_LITERAL(6, 62, 17), // "StartProcessError"
QT_MOC_LITERAL(7, 80, 22), // "QProcess::ProcessError"
QT_MOC_LITERAL(8, 103, 5), // "error"
QT_MOC_LITERAL(9, 109, 13), // "onEngineDeath"
QT_MOC_LITERAL(10, 123, 8), // "exitCode"
QT_MOC_LITERAL(11, 132, 20), // "QProcess::ExitStatus"
QT_MOC_LITERAL(12, 153, 10), // "exitStatus"
QT_MOC_LITERAL(13, 164, 14) // "tcpServerReady"

    },
    "TCPBase\0isReadyNow\0\0NewConnection\0"
    "ClientDisconnect\0ClientRead\0"
    "StartProcessError\0QProcess::ProcessError\0"
    "error\0onEngineDeath\0exitCode\0"
    "QProcess::ExitStatus\0exitStatus\0"
    "tcpServerReady"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TCPBase[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   49,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   50,    2, 0x08 /* Private */,
       4,    0,   51,    2, 0x08 /* Private */,
       5,    0,   52,    2, 0x08 /* Private */,
       6,    1,   53,    2, 0x08 /* Private */,
       9,    2,   56,    2, 0x08 /* Private */,
      13,    0,   61,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 11,   10,   12,
    QMetaType::Void,

       0        // eod
};

void TCPBase::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TCPBase *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->isReadyNow(); break;
        case 1: _t->NewConnection(); break;
        case 2: _t->ClientDisconnect(); break;
        case 3: _t->ClientRead(); break;
        case 4: _t->StartProcessError((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 5: _t->onEngineDeath((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        case 6: _t->tcpServerReady(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TCPBase::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TCPBase::isReadyNow)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TCPBase::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_TCPBase.data,
    qt_meta_data_TCPBase,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TCPBase::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TCPBase::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TCPBase.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int TCPBase::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void TCPBase::isReadyNow()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
