/****************************************************************************
** Meta object code from reading C++ file 'chatwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../QTfrontend/ui/widget/chatwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'chatwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_HWChatWidget_t {
    QByteArrayData data[42];
    char stringdata0[399];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HWChatWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HWChatWidget_t qt_meta_stringdata_HWChatWidget = {
    {
QT_MOC_LITERAL(0, 0, 12), // "HWChatWidget"
QT_MOC_LITERAL(1, 13, 8), // "chatLine"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 3), // "str"
QT_MOC_LITERAL(4, 27, 4), // "kick"
QT_MOC_LITERAL(5, 32, 3), // "ban"
QT_MOC_LITERAL(6, 36, 8), // "delegate"
QT_MOC_LITERAL(7, 45, 4), // "info"
QT_MOC_LITERAL(8, 50, 6), // "follow"
QT_MOC_LITERAL(9, 57, 15), // "nickCountUpdate"
QT_MOC_LITERAL(10, 73, 3), // "cnt"
QT_MOC_LITERAL(11, 77, 14), // "consoleCommand"
QT_MOC_LITERAL(12, 92, 7), // "command"
QT_MOC_LITERAL(13, 100, 12), // "onChatAction"
QT_MOC_LITERAL(14, 113, 4), // "nick"
QT_MOC_LITERAL(15, 118, 13), // "onChatMessage"
QT_MOC_LITERAL(16, 132, 15), // "onServerMessage"
QT_MOC_LITERAL(17, 148, 9), // "nickAdded"
QT_MOC_LITERAL(18, 158, 10), // "notifyNick"
QT_MOC_LITERAL(19, 169, 11), // "nickRemoved"
QT_MOC_LITERAL(20, 181, 7), // "message"
QT_MOC_LITERAL(21, 189, 5), // "clear"
QT_MOC_LITERAL(22, 195, 11), // "adminAccess"
QT_MOC_LITERAL(23, 207, 12), // "onPlayerInfo"
QT_MOC_LITERAL(24, 220, 2), // "ip"
QT_MOC_LITERAL(25, 223, 7), // "version"
QT_MOC_LITERAL(26, 231, 8), // "roomInfo"
QT_MOC_LITERAL(27, 240, 13), // "returnPressed"
QT_MOC_LITERAL(28, 254, 5), // "onBan"
QT_MOC_LITERAL(29, 260, 6), // "onKick"
QT_MOC_LITERAL(30, 267, 10), // "onDelegate"
QT_MOC_LITERAL(31, 278, 6), // "onInfo"
QT_MOC_LITERAL(32, 285, 8), // "onFollow"
QT_MOC_LITERAL(33, 294, 8), // "onIgnore"
QT_MOC_LITERAL(34, 303, 8), // "onFriend"
QT_MOC_LITERAL(35, 312, 21), // "chatNickDoubleClicked"
QT_MOC_LITERAL(36, 334, 11), // "QModelIndex"
QT_MOC_LITERAL(37, 346, 5), // "index"
QT_MOC_LITERAL(38, 352, 11), // "linkClicked"
QT_MOC_LITERAL(39, 364, 4), // "link"
QT_MOC_LITERAL(40, 369, 25), // "nicksContextMenuRequested"
QT_MOC_LITERAL(41, 395, 3) // "pos"

    },
    "HWChatWidget\0chatLine\0\0str\0kick\0ban\0"
    "delegate\0info\0follow\0nickCountUpdate\0"
    "cnt\0consoleCommand\0command\0onChatAction\0"
    "nick\0onChatMessage\0onServerMessage\0"
    "nickAdded\0notifyNick\0nickRemoved\0"
    "message\0clear\0adminAccess\0onPlayerInfo\0"
    "ip\0version\0roomInfo\0returnPressed\0"
    "onBan\0onKick\0onDelegate\0onInfo\0onFollow\0"
    "onIgnore\0onFriend\0chatNickDoubleClicked\0"
    "QModelIndex\0index\0linkClicked\0link\0"
    "nicksContextMenuRequested\0pos"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HWChatWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      28,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  154,    2, 0x06 /* Public */,
       4,    1,  157,    2, 0x06 /* Public */,
       5,    1,  160,    2, 0x06 /* Public */,
       6,    1,  163,    2, 0x06 /* Public */,
       7,    1,  166,    2, 0x06 /* Public */,
       8,    1,  169,    2, 0x06 /* Public */,
       9,    1,  172,    2, 0x06 /* Public */,
      11,    1,  175,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      13,    2,  178,    2, 0x0a /* Public */,
      15,    2,  183,    2, 0x0a /* Public */,
      16,    1,  188,    2, 0x0a /* Public */,
      17,    2,  191,    2, 0x0a /* Public */,
      19,    1,  196,    2, 0x0a /* Public */,
      19,    2,  199,    2, 0x0a /* Public */,
      21,    0,  204,    2, 0x0a /* Public */,
      22,    1,  205,    2, 0x0a /* Public */,
      23,    4,  208,    2, 0x0a /* Public */,
      27,    0,  217,    2, 0x08 /* Private */,
      28,    0,  218,    2, 0x08 /* Private */,
      29,    0,  219,    2, 0x08 /* Private */,
      30,    0,  220,    2, 0x08 /* Private */,
      31,    0,  221,    2, 0x08 /* Private */,
      32,    0,  222,    2, 0x08 /* Private */,
      33,    0,  223,    2, 0x08 /* Private */,
      34,    0,  224,    2, 0x08 /* Private */,
      35,    1,  225,    2, 0x08 /* Private */,
      38,    1,  228,    2, 0x08 /* Private */,
      40,    1,  231,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void, QMetaType::QString,   12,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   14,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   14,    3,
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,   14,   18,
    QMetaType::Void, QMetaType::QString,   14,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,   14,   20,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::QString,   14,   24,   25,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 36,   37,
    QMetaType::Void, QMetaType::QUrl,   39,
    QMetaType::Void, QMetaType::QPoint,   41,

       0        // eod
};

void HWChatWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HWChatWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->chatLine((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->kick((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->ban((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->delegate((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->info((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->follow((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->nickCountUpdate((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->consoleCommand((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->onChatAction((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 9: _t->onChatMessage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 10: _t->onServerMessage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->nickAdded((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 12: _t->nickRemoved((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->nickRemoved((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 14: _t->clear(); break;
        case 15: _t->adminAccess((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->onPlayerInfo((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4]))); break;
        case 17: _t->returnPressed(); break;
        case 18: _t->onBan(); break;
        case 19: _t->onKick(); break;
        case 20: _t->onDelegate(); break;
        case 21: _t->onInfo(); break;
        case 22: _t->onFollow(); break;
        case 23: _t->onIgnore(); break;
        case 24: _t->onFriend(); break;
        case 25: _t->chatNickDoubleClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 26: _t->linkClicked((*reinterpret_cast< const QUrl(*)>(_a[1]))); break;
        case 27: _t->nicksContextMenuRequested((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HWChatWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWChatWidget::chatLine)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (HWChatWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWChatWidget::kick)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (HWChatWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWChatWidget::ban)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (HWChatWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWChatWidget::delegate)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (HWChatWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWChatWidget::info)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (HWChatWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWChatWidget::follow)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (HWChatWidget::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWChatWidget::nickCountUpdate)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (HWChatWidget::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HWChatWidget::consoleCommand)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject HWChatWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_HWChatWidget.data,
    qt_meta_data_HWChatWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HWChatWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HWChatWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HWChatWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int HWChatWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void HWChatWidget::chatLine(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void HWChatWidget::kick(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void HWChatWidget::ban(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void HWChatWidget::delegate(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void HWChatWidget::info(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void HWChatWidget::follow(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void HWChatWidget::nickCountUpdate(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void HWChatWidget::consoleCommand(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
